@@ -282,11 +282,56 @@ match_tree_nodes = function(treeA, treeB, tipsA2B=NULL){
 }
 
 
+get_complement = function(N, indices){
+	include 		 = rep(TRUE, times=N)
+	include[indices] = FALSE
+	return(which(include))
+}
 
 
+# extract the values of independent rates from a transition_matrix, based on a provided index_matrix
+# The index matrix should be as generated by get_transition_index_matrix, i.e. a 2D matrix of size Nstates x Nstates, and values in 1,..,Nrates, where Nrates is the number of independent rate variables
+extract_independent_rates_from_transition_matrix = function(transition_matrix, index_matrix){
+	flattened_index_matrix 	= as.vector(index_matrix)
+	flattened_transition_matrix = as.vector(transition_matrix)
+	independents 			= seq_along(flattened_index_matrix)[!duplicated(flattened_index_matrix)]
+	independents 			= independents[flattened_index_matrix[independents]>0] # omit any zeros from index_matrix
+	independent_rates 		= rep(NA,length(independents))
+	independent_rates[flattened_index_matrix[independents]] = flattened_transition_matrix[independents]
+	return(independent_rates)
+}
 
 
 
+# get a 1D lookup matrix of size Nstates, mapping birth-rates to indices of a subset of independent birth-rates
+# model can be "ER" or "ARD" or a custom index_matrix as if it was generated by this function (in which case it is merely used to determine Nrates)
+# This function is analogous to get_transition_index_matrix(..), but for 1D vectors
+get_rate_index_vector = function(Nstates, rate_model){
+	if (is.character(rate_model)) {
+		if(rate_model == "ER"){
+			Nrates = 1;
+			index_vector = rep(1,Nstates)
+		}else if(rate_model == "ARD"){
+			Nrates = Nstates;
+			index_vector = 1:Nrates;		
+		}else{
+			stop(sprintf("ERROR: Unknown rate_model '%s'",rate_model))
+		}
+	}else{
+		if(length(rate_model)!=Nstates) stop(sprintf("ERROR: Wrong number of elements in rate model (expected %d, found %d)",Nstates,length(rate_model)));
+		index_vector = rate_model
+		Nrates = max(rate_model)
+	}
+	return(list(index_vector=index_vector, Nrates=Nrates))
+}
+
+
+extract_independent_rates_from_vector = function(rates, index_vector){
+	independents 			= seq_along(index_vector)[!duplicated(index_vector)]
+	independent_rates 		= rep(NA,length(independents))
+	independent_rates[index_vector[independents]] = rates[independents]
+	return(independent_rates)
+}
 
 
 
