@@ -27,7 +27,7 @@ fit_musse = function(	tree,
 						NPstates				= NULL,			# optional number of discrete "proxy" states. Each proxy-state may correspond to one or more distinct states. Proxy-states do not directly influence birth/death rates, but each proxy-state represents a set of states. Proxy-states are only needed when defining a "hidden state speciation and extinction" (HiSSE) model [Beaulieu and Meara, 2016]. If this is NULL or 0, then proxy-states are equal to states, and NPstates=Nstates; this is the original MuSSE model (i.e. no latent states).
 						proxy_map				= NULL,			# optional 1D integer vector of size Nstates, mapping states to proxy-states. Hence, proxy_map[s] is an integer in 1:NPstates, specifying which proxy-state the state s belongs to. Only relevant if NPstates!=NULL and NPstates!=Nstates.
 						tip_pstates				= NULL,			# 1D numerical/character/factor array of size Ntips, listing integers in 1:NPstates, specifying the proxy state at each tip. Can also be NULL. May include NA (unknown tip states).
-						tip_priors 				= NULL, 		# 2D numerical array of size Ntips x Nstates, listing the prior likelihood of each state at each tip. Can be provided alternatively to tip_pstates.
+						tip_priors 				= NULL, 		# 2D numerical array of either size Ntips x Nstates or size Ntips x NPstates, listing the prior likelihood of each state (or proxy-state) at each tip. Specifically, tip_priors[i,s] is the likelihood of observing the data, if tip i had state (or proxy-state) s. Can be provided alternatively to tip_pstates.
 						sampling_fractions		= 1,			# optional numerical vector of size NPstates, indicating the sampling fractions (fraction of species included as tips in the tree) conditional upon each species' proxy state. This can be used to incorporate detection biases for species, depending on their proxy state. Can also be NULL or a single number (in which case sampling fractions are assumed to be independent of proxy state).
 						reveal_fractions		= 1,			# optional numerical vector of size NPstates, indicating the resolved fractions (fraction of tree-tips with revealed, i.e. non hidden, state) conditional upon each tip's proxy state. Hence, reveal_fractions[s] is the probability that a species with proxy state s will have revealed state, conditional upon being included in the tree. This can be used to incorporate reveal biases for tips, depending on their proxy state. Can also be NULL or a single number (in which case reveal fractions are assumed to be independent of proxy state).
 						transition_rate_model	= "ARD",		# either "ER" or "SYM" or "ARD" or "SUEDE" or "SRD" or a 2D integer matrix of size Nstates x Nstates mapping entries of the transition matrix to a set of independent transition-rate parameters. The format and interpretation is the same as for index_matrix generated by the function get_transition_index_matrix(..).
@@ -113,34 +113,46 @@ fit_musse = function(	tree,
 		if(!is.numeric(tip_pstates)) stop(sprintf("ERROR: tip_pstates must be integers"))
 		if(length(tip_pstates)==0) stop("ERROR: tip_pstates is non-NULL but empty")
 		if(length(tip_pstates)!=Ntips) stop(sprintf("ERROR: Length of tip_pstates (%d) is not the same as the number of tips in the tree (%d)",length(tip_pstates),Ntips));
-		known_tips = which(!is.na(tip_pstates));
-		if(length(known_tips)==0) stop("ERROR: All tip states are hidden");
+		known_tips  = which(!is.na(tip_pstates));
+		Nknown_tips = length(known_tips)
+		if(Nknown_tips==0) stop("ERROR: All tip states are unspecified");
 		max_tip_state = max(tip_pstates[known_tips])
 		if(check_input){
 			min_tip_state = min(tip_pstates[known_tips])
 			if((min_tip_state<1) || (max_tip_state>NPstates)) stop(sprintf("ERROR: Known tip states must be integers between 1 and %d, but found values between %d and %d",NPstates,min_tip_state,max_tip_state))
 			if((!is.null(names(tip_pstates))) && any(names(tip_pstates)!=tree$tip.label)) stop("ERROR: Names in tip_pstates and tip labels in tree don't match (must be in the same order).")
 		}
+		unknown_tips = get_complement(Ntips, known_tips)
+		# count number of tips in tree (known to be) in each state
+		Nknown_tips_per_pstate = sapply(1:NPstates, function(state) sum(tip_pstates[known_tips]==state));
+		if(verbose && (Nknown_tips<Ntips)) cat(sprintf("%sNote: %d out of %d tips have unspecified state\n",verbose_prefix,Ntips-Nknown_tips,Ntips))
+
 	}else{
 		if(nrow(tip_priors)==0) stop("ERROR: tip_priors is non-NULL but has zero rows")
-		if(Nstates != ncol(tip_priors)) stop(sprintf("ERROR: Nstates (%d) differs from the number of columns in tip_priors (%d)",Nstates,ncol(tip_priors)))
-		known_tips = which(rowSums(is.na(tip_priors))==0);
-		if(length(known_tips)==0) stop("ERROR: All tip states are hidden");
+		if((Nstates != ncol(tip_priors)) && (NPstates != ncol(tip_priors))) stop(sprintf("ERROR: The number of columns in tip_priors (%d) must be equal to Nstates (%d) or NPstates (%d)",ncol(tip_priors),Nstates,NPstates))
+		if(Ntips != nrow(tip_priors)) stop(sprintf("ERROR: Number of tips (%d) differs from the number of rows in tip_priors (%d)",Ntips,nrow(tip_priors)))
+		known_priors = which(rowSums(is.na(tip_priors))==0);
+		if(length(known_priors)==0) stop("ERROR: All tip priors are unspecified");
 		if(check_input){
-			max_tip_prior = max(tip_priors[known_tips,])
+			max_tip_prior = max(tip_priors[known_priors,])
 			if(max_tip_prior>1.0) stop(sprintf("ERROR: Some tip_priors are larger than 1.0 (max was %g)",max_tip_prior))
 			if((!is.null(rownames(tip_priors))) && (!is.null(tree$tip.label)) && (rownames(tip_priors)!=tree$tip.label)) stop("ERROR: Row names in tip_priors and tip labels in tree don't match")
 		}
+		Nknown_priors  = length(known_priors)
+		unknown_priors = get_complement(Ntips, known_priors)
+		if(verbose && (Nknown_priors<Ntips)) cat(sprintf("%sNote: %d out of %d tips have unspecified priors\n",verbose_prefix,Ntips-Nknown_priors,Ntips))
+		# tip states are not explicitly given, but instead we got prior state likelihoods, so calculate number of known tips per pstate using a probabilistic averaging approach
+		# only tips with specified priors are counted; sum(Nknown_tips_per_pstate) will be equal to Nknown_priors
+		if(ncol(tip_priors)==NPstates){
+			Nknown_tips_per_pstate = colSums(tip_priors[known_priors,]/rowSums(tip_priors[known_priors,]))
+		}else{
+			Nknown_tips_per_state  = colSums(tip_priors[known_priors,]/rowSums(tip_priors[known_priors,]))
+			Nknown_tips_per_pstate = sapply(1:NPstates, FUN=function(p) sum(Nknown_tips_per_state[proxy_map==p]))
+		}
 	}
-	Nknown_tips = length(known_tips)
-	unknown_tips = get_complement(Ntips, known_tips)
 	Nstates_per_pstate = sapply(1:NPstates, FUN=function(p) sum(proxy_map==p)) # number of states per proxy-state
 	states_per_pstate  = lapply(1:NPstates, FUN=function(p) which(proxy_map==p)) # states_per_pstate[p] is a 1D vector, listing the states represented by the proxy-state p
-	if(verbose && (Nknown_tips<Ntips)) cat(sprintf("%sNote: %d out of %d tips have unknown state\n",verbose_prefix,Ntips-Nknown_tips,Ntips))
-	
-	# count number of tips in tree (known to be) in each state
-	Nknown_tips_per_pstate = sapply(1:NPstates, function(state) sum(tip_pstates[known_tips]==state));
-	
+		
 	# pre-process sampling_fractions & reveal_fractions
 	if(!has_sampling_fractions){
 		sampling_fractions = rep(1, times=NPstates) # sampling fractions missing, so assume that the phylogeny is complete
@@ -156,7 +168,11 @@ fit_musse = function(	tree,
 		reveal_fractions = reveal_fractions*Ntips/sum(Nknown_tips_per_pstate[Nknown_tips_per_pstate>0]/reveal_fractions[Nknown_tips_per_pstate>0]) # rescale reveal fractions to be consistent with the number of known tips in each state within the considered tree
 	}else{
 		# reveal fractions missing, so assume tips are revealed at the same probabilities (i.e. regardless of state)
-		reveal_fractions = rep(length(known_tips)/Ntips, times=NPstates);
+		if(!is.null(tip_pstates)){
+			reveal_fractions = rep(Nknown_tips/Ntips, times=NPstates);
+		}else{
+			reveal_fractions = rep(Nknown_priors/Ntips, times=NPstates);
+		}
 	}
 	
 	# estimate number of extant species per state
@@ -166,7 +182,7 @@ fit_musse = function(	tree,
 	Nspecies_per_pstate[Nknown_tips_per_pstate>0] = Nknown_tips_per_pstate[Nknown_tips_per_pstate>0]/(reveal_fractions[Nknown_tips_per_pstate>0]*sampling_fractions[Nknown_tips_per_pstate>0]);
 
 	# calculate overall rarefaction of the tree, i.e. fraction of extant species included in the tree
-	overall_rarefaction = Ntips/sum(Nspecies_per_pstate)
+	overall_rarefaction = Ntips/sum(Nspecies_per_pstate)	
 	
 	# if some model parameters were passed as NaN or NULL, replace with NAs
 	# also, if some birth/death rates were passed as single scalars, multiply to Nstates
@@ -199,6 +215,35 @@ fit_musse = function(	tree,
 		root_prior_probabilities = root_prior
 	}
 	
+	# define initial conditions for E & D, depending on sampling fractions & reveal fractions
+	if(is.null(tip_priors)){
+		if(verbose) cat(sprintf("%sPreparing tip priors..\n",verbose_prefix))
+		tip_priors = matrix(0, nrow=Ntips, ncol=Nstates);
+		if(length(known_tips)>0){
+			# for each known tip i, with proxy-state p=tip_pstates[i], set: tip_priors[i,states_per_pstate[p]] = sampling_fractions[p] * reveal_fractions[p]
+			known_tip_pstates 	  = tip_pstates[known_tips]
+			Nstates_per_known_tip = Nstates_per_pstate[known_tip_pstates]
+			tip_priors[cbind(rep(known_tips,times=Nstates_per_known_tip), unlist(states_per_pstate[known_tip_pstates]))] = rep(sampling_fractions[known_tip_pstates] * reveal_fractions[known_tip_pstates], times=Nstates_per_known_tip)
+		}
+		if(length(unknown_tips)>0){
+			likelihood_per_pstate = sampling_fractions*(1-reveal_fractions)
+			tip_priors[unknown_tips,] = matrix(rep(likelihood_per_pstate[proxy_map],times=length(unknown_tips)),nrow=length(unknown_tips),ncol=Nstates,byrow=TRUE)
+		}
+	}else{
+		if(length(unknown_priors)>0){
+			# some tips have unspecified priors, so set to default
+			likelihood_per_pstate = sampling_fractions*(1-reveal_fractions)
+			tip_priors[unknown_priors,] = matrix(rep(likelihood_per_pstate[proxy_map],times=length(unknown_priors)),nrow=length(unknown_priors),ncol=NPstates,byrow=TRUE)	
+		}
+		if(ncol(tip_priors)==NPstates){
+			# tip_priors are given as Ntips x NPstates, so reformat to make sure they are of dimensions Ntips x Nstates
+			# basically, for every state s, set new_tip_priors[,s] = tip_priors[,proxy_map[s]]
+			tip_priors = tip_priors[,proxy_map];
+		}
+	}
+	initial_extinction_probabilities = (1-sampling_fractions[proxy_map]);
+
+	
 	# figure out which parameters are fitted vs fixed
 	# get set of independent parameters (as a condensed vector)
 	# For example, if the birth_rate_model is "ER", only one birth rate is considered an independent model parameter (regardless of whether it is fixed or fitted)
@@ -230,13 +275,10 @@ fit_musse = function(	tree,
 		# some birth_rates and/or some death_rates are non-fixed and have unknown start values, so guesstimate by fitting a birth-death model
 		if(!all(is.na(first_guess$birth_rates))) start_birth_rate_factor = mean(first_guess$birth_rates, na.rm=TRUE)
 		else start_birth_rate_factor = NULL
-		
 		if(!all(is.na(first_guess$death_rates))) start_death_rate_factor = mean(first_guess$death_rates, na.rm=TRUE)
 		else start_death_rate_factor = NULL
-		
 		if(!all(is.na(birth_rates))) birth_rate_factor = mean(birth_rates, na.rm=TRUE)
 		else birth_rate_factor = NULL
-		
 		if(!all(is.na(death_rates))) death_rate_factor = mean(death_rates, na.rm=TRUE)
 		else death_rate_factor = NULL
 
@@ -264,18 +306,17 @@ fit_musse = function(	tree,
 			# fitting failed, so use the birth-death-model start params as a last resort (these are probably reasonable by order of magnitude)
 			first_guess$birth_rates[is.na(first_guess$birth_rates)] = fit$start_parameters$birth_rate_factor
 			first_guess$death_rates[is.na(first_guess$death_rates)] = fit$start_parameters$death_rate_factor		
-		}		
+		}
 	}
 	if(any(is.na(first_guess$transition_matrix))){
 		# some transition rates are non-fixed and have unknown start values, so guesstimate by fitting an Mk model
-		extraction = get_subtree_with_tips(tree, only_tips=known_tips, omit_tips=NULL, collapse_monofurcations=TRUE, force_keep_root=TRUE);
-		subtree = extraction$subtree
-		fit = asr_mk_model(	subtree, 
-							(if(is.null(tip_pstates)) tip_priors[extraction$new2old_tip,] else tip_pstates[extraction$new2old_tip]),
+		fit = hsp_mk_model(	tree, 
+							tip_states				= NULL,
 							Nstates 				= Nstates,
+							tip_priors 				= tip_priors,
 							rate_model 				= transition_rate_model,
-							transition_matrix 		= NULL,
-							include_ancestral_likelihoods = FALSE,
+							transition_matrix		= NULL,
+							include_likelihoods 	= FALSE,
 							root_prior 				= (if((root_prior=="flat") || (root_prior=="empirical")) root_prior else "flat"),
 							Ntrials 				= max(1,Ntrials/5),
 							optim_max_iterations	= optim_max_iterations,
@@ -283,12 +324,29 @@ fit_musse = function(	tree,
 							store_exponentials 		= FALSE,
 							check_input 			= TRUE,
 							Nthreads 				= Nthreads)
+		# OLD CODE. TO BE DELETED
+		#extraction = get_subtree_with_tips(tree, only_tips=known_tips, omit_tips=NULL, collapse_monofurcations=TRUE, force_keep_root=TRUE);
+		#subtree = extraction$subtree
+		#fit = asr_mk_model(	subtree, 
+		#					tip_states 				= NULL,
+		#					Nstates 				= Nstates,
+		#					tip_priors				= tip_priors[extraction$new2old_tip,],
+		#					rate_model 				= transition_rate_model,
+		#					transition_matrix 		= NULL,
+		#					include_ancestral_likelihoods = FALSE,
+		#					root_prior 				= (if((root_prior=="flat") || (root_prior=="empirical")) root_prior else "flat"),
+		#					Ntrials 				= max(1,Ntrials/5),
+		#					optim_max_iterations	= optim_max_iterations,
+		#					optim_rel_tol			= optim_rel_tol,
+		#					store_exponentials 		= FALSE,
+		#					check_input 			= TRUE,
+		#					Nthreads 				= Nthreads)
 		if(fit$success){
 			# fitting succeeded, so use fitted transition rates
 			first_guess$transition_matrix[is.na(first_guess$transition_matrix)] = fit$transition_matrix[is.na(first_guess$transition_matrix)]
 		}else{
 			# fitting failed, so use a rough guesstimate based on the number of tips
-			first_guess$transition_matrix[is.na(first_guess$transition_matrix)] = Nstates/((if(is.null(subtree$edge.length)) 1 else mean(subtree$edge.length))*log(Nknown_tips)/log(2.0))
+			first_guess$transition_matrix[is.na(first_guess$transition_matrix)] = Nstates/((if(is.null(tree$edge.length)) 1 else mean(tree$edge.length))*log(Nknown_tips)/log(2.0))
 		}
 		# make sure first-guess transition matrix is a valid transition matrix
 		diag(first_guess$transition_matrix) = -rowSums(first_guess$transition_matrix);
@@ -327,30 +385,7 @@ fit_musse = function(	tree,
 	param_scales = compress_params(param_scales$transition_matrix, param_scales$birth_rates, param_scales$death_rates, Nstates, transition_indices, birth_rate_indices, death_rate_indices);
 	
 	# pre-calculate node ages, used in loglikelihood function
-	node_ages = get_all_node_depths(tree);
-			
-	# define initial conditions for E & D, depending on sampling fractions & reveal fractions
-	if(verbose) cat(sprintf("%sPreparing objective function for fitting..\n",verbose_prefix))
-	if(!is.null(tip_pstates)){
-		tip_priors = matrix(0, nrow=Ntips, ncol=Nstates);
-		if(length(known_tips)>0){
-			# for each known tip i, with proxy-state p=tip_pstates[i], set: tip_priors[i,states_per_pstate[p]] = sampling_fractions[p] * reveal_fractions[p]
-			known_tip_pstates 	  = tip_pstates[known_tips]
-			Nstates_per_known_tip = Nstates_per_pstate[known_tip_pstates]
-			tip_priors[cbind(rep(known_tips,times=Nstates_per_known_tip), unlist(states_per_pstate[known_tip_pstates]))] = rep(sampling_fractions[known_tip_pstates] * reveal_fractions[known_tip_pstates], times=Nstates_per_known_tip)
-		}
-		if(length(unknown_tips)>0){
-			likelihood_per_pstate = sampling_fractions*(1-reveal_fractions)
-			tip_priors[unknown_tips,] = matrix(rep(likelihood_per_pstate[proxy_map],times=length(unknown_tips)),nrow=length(unknown_tips),ncol=Nstates,byrow=TRUE)
-		}
-	# OLD CODE. TO BE DELETED
-	#}else if(length(unknown_tips)>0){
-		# use provided priors for known_tips as is, but adjust priors of unknown_tips
-		#likelihood_per_pstate = sampling_fractions*(1-reveal_fractions)
-		#tip_priors[unknown_tips,] = matrix(rep(likelihood_per_pstate[proxy_map],times=length(unknown_tips)),nrow=length(unknown_tips),ncol=Nstates,byrow=TRUE)			
-	}
-	initial_extinction_probabilities = (1-sampling_fractions[proxy_map]);
-			
+	node_ages = get_all_node_depths(tree);			
 					
 	# define objective function to be minimized (negated log-likelihood)
 	# the input to the objective function must be scaled and free (non-fixed) independent parametes
@@ -425,7 +460,7 @@ fit_musse = function(	tree,
             Niterations 	= (fit$iterations)
             converged 		= (fit$status==0)
             fit$par 		= fit$solution
-			#fit = subplex::subplex(	par = initial_param_values[fitted_params], 
+			#fit = subplex::subplex(par = initial_param_values[fitted_params], 
 			#						fn = objective_function, 
 			#						control = list(maxit=optim_max_iterations, reltol = optim_rel_tol, parscale=0.1), 
 			#						hessian = FALSE)
